/*
ZigZagUTG - another zig zag indicator
by Alex Matulich, anachronist@gmail.com
Unicorn Research Corporation, April 2012

This is a modified version of the ZigZag algorithm described verbally
on page 39 of _The Ultimate Trading Guide_ by John R. Hill, George
Pruitt, and Lundy Hill published (2000) by John Wiley and Sons, Inc,
ISBN 0-471-38135-7

Instead of using an arbitrary pullback amount to identify a swing high
or a swing low, Hill et al use a more objective definition:
* A swing high bar is the highest high prior to penetration of the low
  of that highest high bar.
* A swing low bar is the lowest low prior to penetration of the high of
  that lowest low bar.

This indicator simplifies the above rule by amending "low of the
highest high bar" to "low of the prior 'Span' bars" where Span is a
sliding window 'Span' bars wide for calculating the recent high or low
to be penetrated.

When Span=1, the indicator behaves pretty much as described in the
book except when a pivot bar is followed by inside bars, in which
case the pivot may be identified earlier than the book's algorithm.
When span=6, this indicator matches Larry Williams' zigzag as
described in his "Money Tree" course.

The RequireCloseBreakout parameter, when true, forces the indicator to
require that a bar's closing price (not just any price) penetrate the
recent high or low of the last 'Span' bars to confirm a new pivot.

This indicator has a number of public properties that can be accessed
from other indicators:

 dir      = current pivot direction (positive = high, negative = low)
 pb1, pb2 = bar numbers of the last two confirmed pivots
 pb3      = bar number of possible new pivot, not yet confirmed
 pv1, pv2 = price values of the last two confirmed pivots
 pv3      = price of the possible new pivot, not yet confirmed
*/
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Collections.Generic; 
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator {
	/// <summary>
	/// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
	/// </summary>
	/// 
	/// 
	
	public class ZigLine {
		public int b1,b2; public double v1,v2;
		public string name;
		public ZigLine(int b1, double v1, int b2, double v2, string name) {
			this.b1 = b1; this.v1 = v1;
			this.b2 = b2;  this.v2 = v2; 
			this.name = name; 
		}
			
	}
	public class ZigLineComparater:IEqualityComparer<ZigLine> {
		public bool Equals(ZigLine line1, ZigLine line2) {
			return String.Equals(line1.name,line2.name);
		}
		public int GetHashCode(ZigLine line1) {
			return line1.name.GetHashCode();
		}
	}
	
	[Description("ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'")]
	public class ZigZagUTG : Indicator {
		#region Variables
		// parameters
		private bool requireCloseBreakout = false;
		private int span = 2;
		private int zzshow = 3;
		// public properties
		public int dir, pb1, pb2, pb3;
		public double pv1, pv2, pv3;
		// other
		private double hival, loval, price;
		private Color plotcolor;
		private int plotwidth;
		private bool drawlines, drawdots;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		/// 
		/// 
		
		HashSet<ZigLine> lines; 
		protected override void Initialize() {
			Add(new Plot(new Pen(Color.CornflowerBlue, 2), PlotStyle.Dot, "Pivot"));
			DrawOnPricePanel = true;
			Overlay = true;
			BarsRequired = 2;
			DisplayInDataBox = false;
			CalculateOnBarClose = false;
			lines = new HashSet<ZigLine>(new ZigLineComparater());
		
		}
		protected override void OnStartUp() {
			plotcolor = Plots[0].Pen.Color;
			plotwidth = (int)Plots[0].Pen.Width;
			if (plotwidth > 3) --plotwidth;
			drawdots = ((zzshow & 1) > 0);
			drawlines = ((zzshow & 2) > 0);
		}

		private int getlobar(IDataSeries x, int span) {
			int barsago = 1;
			double v = x[1];
			for (int i = 2; i <= span; ++i) {
				if (x[i] < v) {
					barsago = i;
					v = x[i];
				}
			}
			return barsago;
		}

		private int gethibar(IDataSeries x, int span) {
			int barsago = 1;
			double v = x[1];
			for (int i = 2; i <= span; ++i) {
				if (x[i] > v) {
					barsago = i;
					v = x[i];
				}
			}
			return barsago;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate() {
			if (CurrentBar < span) return;
			if (CurrentBar == span) {
				if (High[0] > High[span]) {
					dir = 1;
					pv1 = Low[span];
					pv2 = pv3 = High[0];
				} else {
					dir = -1;
					pv1 = High[span];
					pv2 = pv3 = Low[0];
				}
				pb3 = pb2 = 1;
				pb1 = 0;
			}
			if (FirstTickOfBar) {
				hival = High[gethibar(High, span)];
				loval = Low[getlobar(Low, span)];
			} else if (price == Close[0]) return;
			price = Close[0];
			bool tswitch;
			if (dir > 0) {
				tswitch = requireCloseBreakout ? Close[0] < loval : Low[0] < loval;
				if (High[0] > pv3 || (High[0] == pv3 && !tswitch)) {
					if (pb3 < CurrentBar)
						Values[0].Reset(CurrentBar - pb3);
					pb3 = CurrentBar;
					pv3 = High[0];
				} else if (tswitch) {
					pb1 = pb2; pv1 = pv2;
					pb2 = pb3; pv2 = pv3;
					pb3 = CurrentBar;
					pv3 = Low[0];
					dir = -1;
				}
			} else { // dir < 0
				tswitch = requireCloseBreakout ? Close[0] > hival : High[0] > hival;
				if (Low[0] < pv3 || (Low[0] == pv3 && !tswitch)) {
					if (pb3 < CurrentBar)
						Values[0].Reset(CurrentBar - pb3);
					pb3 = CurrentBar;
					pv3 = Low[0];
				} else if (tswitch) {
					pb1 = pb2; pv1 = pv2;
					pb2 = pb3; pv2 = pv3;
					pb3 = CurrentBar;
					pv3 = High[0];
					dir = 1;
				}
			}
			if (pb3 == CurrentBar) {
				if (drawdots) 
					Values[0].Set(pv3);
				else {
					ZigLine line = new ZigLine(pb2,pv2,pb3,pv3,"ZZ"+pb1.ToString());
				
				if (lines.Add(line)==false) {
					lines.Remove(line);
					lines.Add(line);
				}
				}
				//if (drawlines) DrawLine("ZZ"+pb1.ToString(), false, CurrentBar-pb2, pv2, CurrentBar-pb3, pv3, plotcolor, DashStyle.Solid, 2); //plotwidth
			
			}
				
		}
		

		public Point[] getPoint(ZigLine line) {
			int b2 = line.b1;  double v2 = line.v1;
			int b3 = line.b2; double v3 = line.v2;
		
			Point p1 = new Point (ChartControl.GetXByBarIdx(Bars,b2),ChartControl.GetYByValue(this,v2));
			Point p2 = new Point(ChartControl.GetXByBarIdx(Bars,b3),ChartControl.GetYByValue(this,v3));
			return new Point[] {p1,p2};
			
		}
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			base.Plot(graphics,bounds,min,max);
		
			if (Bars == null || ChartControl == null)
				return;
			
			try {
			if (drawlines) {
				//draw all lines in the graphicspath! 
				Pen pen = new Pen(new SolidBrush(plotcolor));
				pen.Width = plotwidth;
				GraphicsPath path = new GraphicsPath();
				foreach (ZigLine line in lines) {
					Point[] points = getPoint(line);
					path.AddLine(points[0],points[1]);
				}
				graphics.DrawPath(pen,path);
	
			}
			} catch (Exception e) {
				Print(e.ToString());
			}
			
		}
		

		#region Properties
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
		[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries Pivot {
			get { return Values[0]; }
		}

		[Description("Require close to break prior high or low to define a pivot")]
		[GridCategory("Parameters")]
		public bool RequireCloseBreakout {
			get { return requireCloseBreakout; }
			set { requireCloseBreakout = value; }
		}
		[Description("Bar span to consider for breakouts")]
		[GridCategory("Parameters")]
		public int Span {
			get { return span; }
			set { span = Math.Max(1, value); }
		}
		[Description("What to display (0=none, 1=dots, 2=lines, 3=both)")]
		[GridCategory("Parameters")]
		public int ZZShow {
			get { return zzshow; }
			set { zzshow = Math.Max(0, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ZigZagUTG[] cacheZigZagUTG = null;

        private static ZigZagUTG checkZigZagUTG = new ZigZagUTG();

        /// <summary>
        /// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
        /// </summary>
        /// <returns></returns>
        public ZigZagUTG ZigZagUTG(bool requireCloseBreakout, int span, int zZShow)
        {
            return ZigZagUTG(Input, requireCloseBreakout, span, zZShow);
        }

        /// <summary>
        /// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
        /// </summary>
        /// <returns></returns>
        public ZigZagUTG ZigZagUTG(Data.IDataSeries input, bool requireCloseBreakout, int span, int zZShow)
        {
            if (cacheZigZagUTG != null)
                for (int idx = 0; idx < cacheZigZagUTG.Length; idx++)
                    if (cacheZigZagUTG[idx].RequireCloseBreakout == requireCloseBreakout && cacheZigZagUTG[idx].Span == span && cacheZigZagUTG[idx].ZZShow == zZShow && cacheZigZagUTG[idx].EqualsInput(input))
                        return cacheZigZagUTG[idx];

            lock (checkZigZagUTG)
            {
                checkZigZagUTG.RequireCloseBreakout = requireCloseBreakout;
                requireCloseBreakout = checkZigZagUTG.RequireCloseBreakout;
                checkZigZagUTG.Span = span;
                span = checkZigZagUTG.Span;
                checkZigZagUTG.ZZShow = zZShow;
                zZShow = checkZigZagUTG.ZZShow;

                if (cacheZigZagUTG != null)
                    for (int idx = 0; idx < cacheZigZagUTG.Length; idx++)
                        if (cacheZigZagUTG[idx].RequireCloseBreakout == requireCloseBreakout && cacheZigZagUTG[idx].Span == span && cacheZigZagUTG[idx].ZZShow == zZShow && cacheZigZagUTG[idx].EqualsInput(input))
                            return cacheZigZagUTG[idx];

                ZigZagUTG indicator = new ZigZagUTG();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.RequireCloseBreakout = requireCloseBreakout;
                indicator.Span = span;
                indicator.ZZShow = zZShow;
                Indicators.Add(indicator);
                indicator.SetUp();

                ZigZagUTG[] tmp = new ZigZagUTG[cacheZigZagUTG == null ? 1 : cacheZigZagUTG.Length + 1];
                if (cacheZigZagUTG != null)
                    cacheZigZagUTG.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheZigZagUTG = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ZigZagUTG ZigZagUTG(bool requireCloseBreakout, int span, int zZShow)
        {
            return _indicator.ZigZagUTG(Input, requireCloseBreakout, span, zZShow);
        }

        /// <summary>
        /// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
        /// </summary>
        /// <returns></returns>
        public Indicator.ZigZagUTG ZigZagUTG(Data.IDataSeries input, bool requireCloseBreakout, int span, int zZShow)
        {
            return _indicator.ZigZagUTG(input, requireCloseBreakout, span, zZShow);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ZigZagUTG ZigZagUTG(bool requireCloseBreakout, int span, int zZShow)
        {
            return _indicator.ZigZagUTG(Input, requireCloseBreakout, span, zZShow);
        }

        /// <summary>
        /// ZigZag indicator inspired by John Hill's 'Ultimate Trading Guide'
        /// </summary>
        /// <returns></returns>
        public Indicator.ZigZagUTG ZigZagUTG(Data.IDataSeries input, bool requireCloseBreakout, int span, int zZShow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ZigZagUTG(input, requireCloseBreakout, span, zZShow);
        }
    }
}
#endregion
